//**********************************************************************`
//* This is an include file generated by Message Compiler.             *`
//*                                                                    *`
//* Copyright (c) Microsoft Corporation. All Rights Reserved.          *`
//**********************************************************************`
#pragma once

//*****************************************************************************
//
// Notes on the ETW event code generated by MC:
//
// - Structures and arrays of structures are treated as an opaque binary blob.
//   The caller is responsible for packing the data for the structure into a
//   single region of memory, with no padding between values. The macro will
//   have an extra parameter for the length of the blob.
// - Arrays of nul-terminated strings must be packed by the caller into a
//   single binary blob containing the correct number of strings, with a nul
//   after each string. The size of the blob is specified in characters, and
//   includes the final nul.
// - Arrays of SID are treated as a single binary blob. The caller is
//   responsible for packing the SID values into a single region of memory with
//   no padding.
// - The length attribute on the data element in the manifest is significant
//   for values with intype win:UnicodeString, win:AnsiString, or win:Binary.
//   The length attribute must be specified for win:Binary, and is optional for
//   win:UnicodeString and win:AnsiString (if no length is given, the strings
//   are assumed to be nul-terminated). For win:UnicodeString, the length is
//   measured in characters, not bytes.
// - For an array of win:UnicodeString, win:AnsiString, or win:Binary, the
//   length attribute applies to every value in the array, so every value in
//   the array must have the same length. The values in the array are provided
//   to the macro via a single pointer -- the caller is responsible for packing
//   all of the values into a single region of memory with no padding between
//   values.
// - Values of type win:CountedUnicodeString, win:CountedAnsiString, and
//   win:CountedBinary can be generated and collected on Vista or later.
//   However, they may not decode properly without the Windows 10 2018 Fall
//   Update.
// - Arrays of type win:CountedUnicodeString, win:CountedAnsiString, and
//   win:CountedBinary must be packed by the caller into a single region of
//   memory. The format for each item is a UINT16 byte-count followed by that
//   many bytes of data. When providing the array to the generated macro, you
//   must provide the total size of the packed array data, including the UINT16
//   sizes for each item. In the case of win:CountedUnicodeString, the data
//   size is specified in WCHAR (16-bit) units. In the case of
//   win:CountedAnsiString and win:CountedBinary, the data size is specified in
//   bytes.
//
//*****************************************************************************

#include <wmistr.h>
#include <evntrace.h>
#include <evntprov.h>

#ifndef ETW_INLINE
  #ifdef _ETW_KM_
    // In kernel mode, save stack space by never inlining templates.
    #define ETW_INLINE DECLSPEC_NOINLINE __inline
  #else
    // In user mode, save code size by inlining templates as appropriate.
    #define ETW_INLINE __inline
  #endif
#endif // ETW_INLINE

#if defined(__cplusplus)
extern "C" {
#endif

//
// MCGEN_DISABLE_PROVIDER_CODE_GENERATION macro:
// Define this macro to have the compiler skip the generated functions in this
// header.
//

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Provider "Microsoft-Windows-Direct3D12" event count 146
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// Provider GUID = 5d8087dd-3a9b-4f56-90df-49196cdc4f11
EXTERN_C __declspec(selectany) const GUID Direct3D12EtwProviderGuid = {0x5d8087dd, 0x3a9b, 0x4f56, {0x90, 0xdf, 0x49, 0x19, 0x6c, 0xdc, 0x4f, 0x11}};

#ifndef Direct3D12EtwProviderGuid_Traits
#define Direct3D12EtwProviderGuid_Traits NULL
#endif // Direct3D12EtwProviderGuid_Traits

//
// Channel
//
#define Direct3D12AnalyticEtwChannel 0x10
#define Direct3D12AnalyticEtwChannel_KEYWORD 0x8000000000000000
#define Direct3D12LoggingEtwChannel 0x11
#define Direct3D12LoggingEtwChannel_KEYWORD 0x4000000000000000
#define Direct3D12PerfTimingEtwChannel 0x12
#define Direct3D12PerfTimingEtwChannel_KEYWORD 0x2000000000000000

//
// Tasks
//
#define Direct3D12EtwProviderGuid_TASK_Name 0x1
#define Direct3D12EtwProviderGuid_TASK_Device 0x2
#define Direct3D12EtwProviderGuid_TASK_D3DSupplemental 0xb
#define Direct3D12EtwProviderGuid_TASK_Destruction 0xf
#define Direct3D12EtwProviderGuid_TASK_Marker 0x14
#define Direct3D12EtwProviderGuid_TASK_RuntimeMarker 0x15
#define Direct3D12EtwProviderGuid_TASK_RuntimeDroppedCall 0x16
#define Direct3D12EtwProviderGuid_TASK_CustomDriverMarker 0x17
#define Direct3D12EtwProviderGuid_TASK_CommandQueue 0x18
#define Direct3D12EtwProviderGuid_TASK_CommandList 0x19
#define Direct3D12EtwProviderGuid_TASK_ExecuteCommandList 0x1a
#define Direct3D12EtwProviderGuid_TASK_Fence 0x1b
#define Direct3D12EtwProviderGuid_TASK_CommandAllocator 0x1c
#define Direct3D12EtwProviderGuid_TASK_GraphicsPipelineState 0x1d
#define Direct3D12EtwProviderGuid_TASK_DescriptorHeap 0x1e
#define Direct3D12EtwProviderGuid_TASK_RootSignature 0x1f
#define Direct3D12EtwProviderGuid_TASK_Heap 0x20
#define Direct3D12EtwProviderGuid_TASK_QueryHeap 0x21
#define Direct3D12EtwProviderGuid_TASK_CommandSignature 0x22
#define Direct3D12EtwProviderGuid_TASK_PipelineLibrary 0x23
#define Direct3D12EtwProviderGuid_TASK_VideoDecoder 0x24
#define Direct3D12EtwProviderGuid_TASK_VideoProcessor 0x25
#define Direct3D12EtwProviderGuid_TASK_Resource 0x26
#define Direct3D12EtwProviderGuid_TASK_AllocationInfo 0x27
#define Direct3D12EtwProviderGuid_TASK_CommandAllocatorUnacquire 0x28
#define Direct3D12EtwProviderGuid_TASK_VideoDecoderHeap 0x29
#define Direct3D12EtwProviderGuid_TASK_ExtendedFeatureVersion 0x2a
#define Direct3D12EtwProviderGuid_TASK_JournalEntry 0x2b
#define Direct3D12EtwProviderGuid_TASK_CommandPool 0x2c
#define Direct3D12EtwProviderGuid_TASK_CommandRecorder 0x2d
#define Direct3D12EtwProviderGuid_TASK_CommandRecorderAcquire 0x2e
#define Direct3D12EtwProviderGuid_TASK_CommandRecorderUnacquire 0x2f
#define Direct3D12EtwProviderGuid_TASK_CreateHeapSharedHandle 0x30
#define Direct3D12EtwProviderGuid_TASK_StateObject 0x31
#define Direct3D12EtwProviderGuid_TASK_MetaCommand 0x32
#define Direct3D12EtwProviderGuid_TASK_ProtectedResourceSession 0x33
#define Direct3D12EtwProviderGuid_TASK_LifetimeTracker 0x34
#define Direct3D12EtwProviderGuid_TASK_SchedulingGroup 0x35
#define Direct3D12EtwProviderGuid_TASK_VideoMotionEstimator 0x36
#define Direct3D12EtwProviderGuid_TASK_VideoMotionVectorHeap 0x37
#define Direct3D12EtwProviderGuid_TASK_DecodePictureLists 0x38
#define Direct3D12EtwProviderGuid_TASK_BackgroundWork 0x39
#define Direct3D12EtwProviderGuid_TASK_VideoExtensionCommand 0x3a
#define Direct3D12EtwProviderGuid_TASK_HistoryBufferCompletion 0x3b
#define Direct3D12EtwProviderGuid_TASK_TrackWrittenResource 0x3c
#define Direct3D12EtwProviderGuid_TASK_ShaderCacheSession 0x3d
#define Direct3D12EtwProviderGuid_TASK_DRED 0x3e
#define Direct3D12EtwProviderGuid_TASK_TrackedWorkload 0x3f
#define Direct3D12EtwProviderGuid_TASK_VideoEncoder 0x40
#define Direct3D12EtwProviderGuid_TASK_VideoEncoderHeap 0x41
#define Direct3D12EtwProviderGuid_TASK_CreatePipelineStateObject 0x42
#define Direct3D12EtwProviderGuid_TASK_CreateStateObject 0x43
#define Direct3D12EtwProviderGuid_TASK_AddToStateObject 0x44
#define Direct3D12EtwProviderGuid_TASK_AddToSODB 0x45
#define Direct3D12EtwProviderGuid_TASK_VideoEncodeFrame 0x46
#define Direct3D12EtwProviderGuid_TASK_VideoDecodeFrame 0x47
#define Direct3D12EtwProviderGuid_TASK_VideoProcessFrame 0x48

//
// Keyword
//
#define D3D12_ETW_LOG_FLAGS_NAMES 0x1
#define D3D12_ETW_LOG_FLAGS_DEVICES 0x2
#define D3D12_ETW_LOG_FLAGS_RESOURCES 0x4
#define D3D12_ETW_LOG_FLAGS_MARKERS 0x8
#define D3D12_ETW_LOG_FLAGS_RENDER_OPERATION_MARKERS 0x10
#define D3D12_ETW_LOG_FLAGS_DRIVER_CUSTOM_MARKERS 0x20
#define D3D12_ETW_LOG_FLAGS_BUNDLE_MARKERS 0x40
#define D3D12_ETW_LOG_FLAGS_OBJECT_LIFETIME 0x80
#define D3D12_ETW_LOG_FLAGS_JOURNAL_ENTRIES 0x100
#define D3D12_ETW_LOG_DECODE_INFO 0x200
#define D3D12_ETW_LOG_APIS 0x400
#define D3D12_ETW_LOG_SODB 0x800
#define D3D12_ETW_LOG_FLAGS_MINIMAL_MARKERS 0x1000
#define D3D12_ETW_LOG_FLAGS_VIDEO_COMMAND_LISTS 0x2000

//
// Event Descriptors
//
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12RenameObject = {0x1, 0x0, 0x10, 0x0, 0x0, 0x1, 0x8000000000000001};
#define EventD3D12RenameObject_value 0x1
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportName = {0x2, 0x0, 0x10, 0x0, 0x3, 0x1, 0x8000000000000001};
#define EventD3D12ReportName_value 0x2
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateDevice = {0x3, 0x1, 0x10, 0x0, 0x1, 0x2, 0x8000000000000082};
#define EventD3D12CreateDevice_value 0x3
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyDevice = {0x4, 0x1, 0x10, 0x0, 0x2, 0x2, 0x8000000000000082};
#define EventD3D12DestroyDevice_value 0x4
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportDevice = {0x5, 0x1, 0x10, 0x0, 0x3, 0x2, 0x8000000000000082};
#define EventD3D12ReportDevice_value 0x5
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12FlipSemaphoreTimeout = {0x13, 0x0, 0x10, 0x0, 0x0, 0x2, 0x8000000000000002};
#define EventD3D12FlipSemaphoreTimeout_value 0x13
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR D3D12SupplementalContiguous = {0x1a, 0x0, 0x11, 0x0, 0x0, 0xb, 0x4000000000000000};
#define D3D12SupplementalContiguous_value 0x1a
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CommandQueueSetMarker = {0x23, 0x0, 0x10, 0x0, 0x0, 0x14, 0x8000000000000008};
#define EventD3D12CommandQueueSetMarker_value 0x23
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CommandQueueBeginMarker = {0x24, 0x0, 0x10, 0x0, 0x1, 0x14, 0x8000000000000008};
#define EventD3D12CommandQueueBeginMarker_value 0x24
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CommandQueueEndMarker = {0x25, 0x0, 0x10, 0x0, 0x2, 0x14, 0x8000000000000008};
#define EventD3D12CommandQueueEndMarker_value 0x25
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12Marker = {0x26, 0x0, 0x10, 0x0, 0x0, 0x14, 0x8000000000000008};
#define EventD3D12Marker_value 0x26
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12RuntimeMarkerData = {0x27, 0x0, 0x10, 0x0, 0x0, 0x15, 0x8000000000000010};
#define EventD3D12RuntimeMarkerData_value 0x27
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12RuntimeDroppedCall = {0x28, 0x0, 0x10, 0x0, 0x0, 0x16, 0x8000000000000010};
#define EventD3D12RuntimeDroppedCall_value 0x28
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CommandBufferSubmission = {0x29, 0x1, 0x10, 0x0, 0x0, 0x15, 0x8000000000000010};
#define EventD3D12CommandBufferSubmission_value 0x29
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CustomDriverMarker = {0x2d, 0x0, 0x10, 0x0, 0x0, 0x17, 0x8000000000000020};
#define EventD3D12CustomDriverMarker_value 0x2d
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateCommandQueue = {0x2f, 0x2, 0x10, 0x0, 0x1, 0x18, 0x8000000000000082};
#define EventD3D12CreateCommandQueue_value 0x2f
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyCommandQueue = {0x30, 0x2, 0x10, 0x0, 0x2, 0x18, 0x8000000000000082};
#define EventD3D12DestroyCommandQueue_value 0x30
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportCommandQueue = {0x31, 0x2, 0x10, 0x0, 0x3, 0x18, 0x8000000000000082};
#define EventD3D12ReportCommandQueue_value 0x31
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateCommandList = {0x32, 0x0, 0x10, 0x0, 0x1, 0x19, 0x8000000000000082};
#define EventD3D12CreateCommandList_value 0x32
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyCommandList = {0x33, 0x0, 0x10, 0x0, 0x2, 0x19, 0x8000000000000082};
#define EventD3D12DestroyCommandList_value 0x33
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ResetCommandList = {0x34, 0x0, 0x10, 0x0, 0x0, 0x19, 0x8000000000000002};
#define EventD3D12ResetCommandList_value 0x34
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12StartExecuteCommandList = {0x35, 0x0, 0x10, 0x0, 0x1, 0x1a, 0x8000000000000002};
#define EventD3D12StartExecuteCommandList_value 0x35
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12StopExecuteCommandList = {0x36, 0x0, 0x10, 0x0, 0x2, 0x1a, 0x8000000000000002};
#define EventD3D12StopExecuteCommandList_value 0x36
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12RenameObjectWide = {0x37, 0x0, 0x10, 0x0, 0x0, 0x1, 0x8000000000000001};
#define EventD3D12RenameObjectWide_value 0x37
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportNameWide = {0x38, 0x0, 0x10, 0x0, 0x3, 0x1, 0x8000000000000001};
#define EventD3D12ReportNameWide_value 0x38
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateFence = {0x39, 0x0, 0x10, 0x0, 0x1, 0x1b, 0x8000000000000084};
#define EventD3D12CreateFence_value 0x39
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyFence = {0x3a, 0x0, 0x10, 0x0, 0x2, 0x1b, 0x8000000000000084};
#define EventD3D12DestroyFence_value 0x3a
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportFence = {0x3b, 0x0, 0x10, 0x0, 0x3, 0x1b, 0x8000000000000084};
#define EventD3D12ReportFence_value 0x3b
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateCommandAllocator = {0x3c, 0x0, 0x10, 0x0, 0x1, 0x1c, 0x8000000000000084};
#define EventD3D12CreateCommandAllocator_value 0x3c
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyCommandAllocator = {0x3d, 0x0, 0x10, 0x0, 0x2, 0x1c, 0x8000000000000084};
#define EventD3D12DestroyCommandAllocator_value 0x3d
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportCommandAllocator = {0x3e, 0x0, 0x10, 0x0, 0x3, 0x1c, 0x8000000000000084};
#define EventD3D12ReportCommandAllocator_value 0x3e
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateGraphicsPipelineState = {0x3f, 0x0, 0x10, 0x0, 0x1, 0x1d, 0x8000000000000084};
#define EventD3D12CreateGraphicsPipelineState_value 0x3f
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyGraphicsPipelineState = {0x40, 0x0, 0x10, 0x0, 0x2, 0x1d, 0x8000000000000084};
#define EventD3D12DestroyGraphicsPipelineState_value 0x40
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportGraphicsPipelineState = {0x41, 0x0, 0x10, 0x0, 0x3, 0x1d, 0x8000000000000084};
#define EventD3D12ReportGraphicsPipelineState_value 0x41
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateDescriptorHeap = {0x42, 0x0, 0x10, 0x0, 0x1, 0x1e, 0x8000000000000084};
#define EventD3D12CreateDescriptorHeap_value 0x42
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyDescriptorHeap = {0x43, 0x0, 0x10, 0x0, 0x2, 0x1e, 0x8000000000000084};
#define EventD3D12DestroyDescriptorHeap_value 0x43
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportDescriptorHeap = {0x44, 0x0, 0x10, 0x0, 0x3, 0x1e, 0x8000000000000084};
#define EventD3D12ReportDescriptorHeap_value 0x44
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateRootSignature = {0x45, 0x0, 0x10, 0x0, 0x1, 0x1f, 0x8000000000000084};
#define EventD3D12CreateRootSignature_value 0x45
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyRootSignature = {0x46, 0x0, 0x10, 0x0, 0x2, 0x1f, 0x8000000000000084};
#define EventD3D12DestroyRootSignature_value 0x46
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportRootSignature = {0x47, 0x0, 0x10, 0x0, 0x3, 0x1f, 0x8000000000000084};
#define EventD3D12ReportRootSignature_value 0x47
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateHeap = {0x48, 0x0, 0x10, 0x0, 0x1, 0x20, 0x8000000000000084};
#define EventD3D12CreateHeap_value 0x48
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyHeap = {0x49, 0x0, 0x10, 0x0, 0x2, 0x20, 0x8000000000000084};
#define EventD3D12DestroyHeap_value 0x49
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportHeap = {0x4a, 0x0, 0x10, 0x0, 0x3, 0x20, 0x8000000000000084};
#define EventD3D12ReportHeap_value 0x4a
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateQueryHeap = {0x4b, 0x0, 0x10, 0x0, 0x1, 0x21, 0x8000000000000084};
#define EventD3D12CreateQueryHeap_value 0x4b
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyQueryHeap = {0x4c, 0x0, 0x10, 0x0, 0x2, 0x21, 0x8000000000000084};
#define EventD3D12DestroyQueryHeap_value 0x4c
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportQueryHeap = {0x4d, 0x0, 0x10, 0x0, 0x3, 0x21, 0x8000000000000084};
#define EventD3D12ReportQueryHeap_value 0x4d
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateCommandSignature = {0x4e, 0x0, 0x10, 0x0, 0x1, 0x22, 0x8000000000000084};
#define EventD3D12CreateCommandSignature_value 0x4e
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyCommandSignature = {0x4f, 0x0, 0x10, 0x0, 0x2, 0x22, 0x8000000000000084};
#define EventD3D12DestroyCommandSignature_value 0x4f
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportCommandSignature = {0x50, 0x0, 0x10, 0x0, 0x3, 0x22, 0x8000000000000084};
#define EventD3D12ReportCommandSignature_value 0x50
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreatePipelineLibrary = {0x51, 0x0, 0x10, 0x0, 0x1, 0x23, 0x8000000000000084};
#define EventD3D12CreatePipelineLibrary_value 0x51
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyPipelineLibrary = {0x52, 0x0, 0x10, 0x0, 0x2, 0x23, 0x8000000000000084};
#define EventD3D12DestroyPipelineLibrary_value 0x52
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportPipelineLibrary = {0x53, 0x0, 0x10, 0x0, 0x3, 0x23, 0x8000000000000084};
#define EventD3D12ReportPipelineLibrary_value 0x53
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateVideoDecoder = {0x54, 0x0, 0x10, 0x0, 0x1, 0x24, 0x8000000000000084};
#define EventD3D12CreateVideoDecoder_value 0x54
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyVideoDecoder = {0x55, 0x0, 0x10, 0x0, 0x2, 0x24, 0x8000000000000084};
#define EventD3D12DestroyVideoDecoder_value 0x55
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportVideoDecoder = {0x56, 0x0, 0x10, 0x0, 0x3, 0x24, 0x8000000000000084};
#define EventD3D12ReportVideoDecoder_value 0x56
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateVideoProcessor = {0x57, 0x0, 0x10, 0x0, 0x1, 0x25, 0x8000000000000084};
#define EventD3D12CreateVideoProcessor_value 0x57
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyVideoProcessor = {0x58, 0x0, 0x10, 0x0, 0x2, 0x25, 0x8000000000000084};
#define EventD3D12DestroyVideoProcessor_value 0x58
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportVideoProcessor = {0x59, 0x0, 0x10, 0x0, 0x3, 0x25, 0x8000000000000084};
#define EventD3D12ReportVideoProcessor_value 0x59
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportCommandList = {0x5a, 0x0, 0x10, 0x0, 0x3, 0x19, 0x8000000000000084};
#define EventD3D12ReportCommandList_value 0x5a
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateResource = {0x5b, 0x1, 0x10, 0x0, 0x1, 0x26, 0x8000000000000084};
#define EventD3D12CreateResource_value 0x5b
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyResource = {0x5c, 0x1, 0x10, 0x0, 0x2, 0x26, 0x8000000000000084};
#define EventD3D12DestroyResource_value 0x5c
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportResource = {0x5d, 0x1, 0x10, 0x0, 0x3, 0x26, 0x8000000000000084};
#define EventD3D12ReportResource_value 0x5d
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12AllocationInfo = {0x5e, 0x1, 0x10, 0x0, 0x0, 0x27, 0x8000000000000004};
#define EventD3D12AllocationInfo_value 0x5e
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CommandAllocatorUnacquire = {0x5f, 0x0, 0x10, 0x0, 0x0, 0x28, 0x8000000000000004};
#define EventD3D12CommandAllocatorUnacquire_value 0x5f
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateVideoDecoderHeap = {0x60, 0x0, 0x10, 0x0, 0x1, 0x29, 0x8000000000000084};
#define EventD3D12CreateVideoDecoderHeap_value 0x60
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyVideoDecoderHeap = {0x61, 0x0, 0x10, 0x0, 0x2, 0x29, 0x8000000000000084};
#define EventD3D12DestroyVideoDecoderHeap_value 0x61
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportVideoDecoderHeap = {0x62, 0x0, 0x10, 0x0, 0x3, 0x29, 0x8000000000000084};
#define EventD3D12ReportVideoDecoderHeap_value 0x62
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ExtendedFeatureVersion = {0x63, 0x0, 0x10, 0x0, 0x0, 0x2a, 0x8000000000000082};
#define EventD3D12ExtendedFeatureVersion_value 0x63
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12JournalEntry = {0x64, 0x0, 0x10, 0x0, 0x0, 0x2b, 0x8000000000000102};
#define EventD3D12JournalEntry_value 0x64
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateCommandPool = {0x65, 0x0, 0x10, 0x0, 0x1, 0x2c, 0x8000000000000084};
#define EventD3D12CreateCommandPool_value 0x65
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyCommandPool = {0x66, 0x0, 0x10, 0x0, 0x2, 0x2c, 0x8000000000000084};
#define EventD3D12DestroyCommandPool_value 0x66
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportCommandPool = {0x67, 0x0, 0x10, 0x0, 0x3, 0x2c, 0x8000000000000084};
#define EventD3D12ReportCommandPool_value 0x67
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateCommandRecorder = {0x68, 0x0, 0x10, 0x0, 0x1, 0x2d, 0x8000000000000084};
#define EventD3D12CreateCommandRecorder_value 0x68
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyCommandRecorder = {0x69, 0x0, 0x10, 0x0, 0x2, 0x2d, 0x8000000000000084};
#define EventD3D12DestroyCommandRecorder_value 0x69
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportCommandRecorder = {0x6a, 0x0, 0x10, 0x0, 0x3, 0x2d, 0x8000000000000084};
#define EventD3D12ReportCommandRecorder_value 0x6a
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CommandRecorderAcquire = {0x6b, 0x0, 0x10, 0x0, 0x1, 0x2e, 0x8000000000000004};
#define EventD3D12CommandRecorderAcquire_value 0x6b
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CommandRecorderUnacquire = {0x6c, 0x0, 0x10, 0x0, 0x2, 0x2f, 0x8000000000000004};
#define EventD3D12CommandRecorderUnacquire_value 0x6c
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateHeapSharedHandle = {0x6d, 0x0, 0x10, 0x0, 0x0, 0x30, 0x8000000000000004};
#define EventD3D12CreateHeapSharedHandle_value 0x6d
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateStateObject = {0x6e, 0x0, 0x10, 0x0, 0x1, 0x31, 0x8000000000000084};
#define EventD3D12CreateStateObject_value 0x6e
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyStateObject = {0x6f, 0x0, 0x10, 0x0, 0x2, 0x31, 0x8000000000000084};
#define EventD3D12DestroyStateObject_value 0x6f
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportStateObject = {0x70, 0x0, 0x10, 0x0, 0x3, 0x31, 0x8000000000000084};
#define EventD3D12ReportStateObject_value 0x70
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateMetaCommand = {0x71, 0x0, 0x10, 0x0, 0x1, 0x32, 0x8000000000000084};
#define EventD3D12CreateMetaCommand_value 0x71
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyMetaCommand = {0x72, 0x0, 0x10, 0x0, 0x2, 0x32, 0x8000000000000084};
#define EventD3D12DestroyMetaCommand_value 0x72
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportMetaCommand = {0x73, 0x0, 0x10, 0x0, 0x3, 0x32, 0x8000000000000084};
#define EventD3D12ReportMetaCommand_value 0x73
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateProtectedResourceSession = {0x74, 0x0, 0x10, 0x0, 0x1, 0x33, 0x8000000000000084};
#define EventD3D12CreateProtectedResourceSession_value 0x74
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyProtectedResourceSession = {0x75, 0x0, 0x10, 0x0, 0x2, 0x33, 0x8000000000000084};
#define EventD3D12DestroyProtectedResourceSession_value 0x75
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportProtectedResourceSession = {0x76, 0x0, 0x10, 0x0, 0x3, 0x33, 0x8000000000000084};
#define EventD3D12ReportProtectedResourceSession_value 0x76
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateLifetimeTracker = {0x77, 0x0, 0x10, 0x0, 0x1, 0x34, 0x8000000000000084};
#define EventD3D12CreateLifetimeTracker_value 0x77
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyLifetimeTracker = {0x78, 0x0, 0x10, 0x0, 0x2, 0x34, 0x8000000000000084};
#define EventD3D12DestroyLifetimeTracker_value 0x78
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportLifetimeTracker = {0x79, 0x0, 0x10, 0x0, 0x3, 0x34, 0x8000000000000084};
#define EventD3D12ReportLifetimeTracker_value 0x79
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateSchedulingGroup = {0x7a, 0x1, 0x10, 0x0, 0x1, 0x35, 0x8000000000000084};
#define EventD3D12CreateSchedulingGroup_value 0x7a
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroySchedulingGroup = {0x7b, 0x1, 0x10, 0x0, 0x2, 0x35, 0x8000000000000084};
#define EventD3D12DestroySchedulingGroup_value 0x7b
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportSchedulingGroup = {0x7c, 0x1, 0x10, 0x0, 0x3, 0x35, 0x8000000000000084};
#define EventD3D12ReportSchedulingGroup_value 0x7c
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateVideoMotionEstimator = {0x7d, 0x0, 0x10, 0x0, 0x1, 0x36, 0x8000000000000084};
#define EventD3D12CreateVideoMotionEstimator_value 0x7d
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyVideoMotionEstimator = {0x7e, 0x0, 0x10, 0x0, 0x2, 0x36, 0x8000000000000084};
#define EventD3D12DestroyVideoMotionEstimator_value 0x7e
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportVideoMotionEstimator = {0x7f, 0x0, 0x10, 0x0, 0x3, 0x36, 0x8000000000000084};
#define EventD3D12ReportVideoMotionEstimator_value 0x7f
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateVideoMotionVectorHeap = {0x80, 0x0, 0x10, 0x0, 0x1, 0x37, 0x8000000000000084};
#define EventD3D12CreateVideoMotionVectorHeap_value 0x80
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyVideoMotionVectorHeap = {0x81, 0x0, 0x10, 0x0, 0x2, 0x37, 0x8000000000000084};
#define EventD3D12DestroyVideoMotionVectorHeap_value 0x81
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportVideoMotionVectorHeap = {0x82, 0x0, 0x10, 0x0, 0x3, 0x37, 0x8000000000000084};
#define EventD3D12ReportVideoMotionVectorHeap_value 0x82
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DecodePictureLists = {0x83, 0x1, 0x10, 0x0, 0x0, 0x38, 0x8000000000000200};
#define EventD3D12DecodePictureLists_value 0x83
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12BeginBackgroundWork = {0x84, 0x0, 0x10, 0x0, 0x1, 0x39, 0x8000000000000008};
#define EventD3D12BeginBackgroundWork_value 0x84
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12EndBackgroundWork = {0x85, 0x0, 0x10, 0x0, 0x2, 0x39, 0x8000000000000008};
#define EventD3D12EndBackgroundWork_value 0x85
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CancelBackgroundWork = {0x86, 0x0, 0x10, 0x0, 0x0, 0x39, 0x8000000000000008};
#define EventD3D12CancelBackgroundWork_value 0x86
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateVideoExtensionCommand = {0x87, 0x0, 0x10, 0x0, 0x1, 0x3a, 0x8000000000000084};
#define EventD3D12CreateVideoExtensionCommand_value 0x87
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyVideoExtensionCommand = {0x88, 0x0, 0x10, 0x0, 0x2, 0x3a, 0x8000000000000084};
#define EventD3D12DestroyVideoExtensionCommand_value 0x88
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportVideoExtensionCommand = {0x89, 0x0, 0x10, 0x0, 0x3, 0x3a, 0x8000000000000084};
#define EventD3D12ReportVideoExtensionCommand_value 0x89
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12HistoryBufferCompletion = {0x8a, 0x0, 0x10, 0x0, 0x0, 0x3b, 0x8000000000000010};
#define EventD3D12HistoryBufferCompletion_value 0x8a
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12StartExecuteCommandLists = {0x8b, 0x0, 0x10, 0x0, 0x1, 0x1a, 0x8000000000000002};
#define EventD3D12StartExecuteCommandLists_value 0x8b
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12StopExecuteCommandLists = {0x8c, 0x0, 0x10, 0x0, 0x2, 0x1a, 0x8000000000000002};
#define EventD3D12StopExecuteCommandLists_value 0x8c
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12TrackWrittenResource = {0x8d, 0x0, 0x10, 0x0, 0x0, 0x3c, 0x8000000000000004};
#define EventD3D12TrackWrittenResource_value 0x8d
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateShaderCacheSession = {0x8e, 0x0, 0x10, 0x0, 0x1, 0x3d, 0x8000000000000080};
#define EventD3D12CreateShaderCacheSession_value 0x8e
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyShaderCacheSession = {0x8f, 0x0, 0x10, 0x0, 0x2, 0x3d, 0x8000000000000080};
#define EventD3D12DestroyShaderCacheSession_value 0x8f
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportShaderCacheSession = {0x90, 0x0, 0x10, 0x0, 0x3, 0x3d, 0x8000000000000080};
#define EventD3D12ReportShaderCacheSession_value 0x90
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12AutoBreadcrumbEntry = {0x91, 0x0, 0x10, 0x0, 0x0, 0x3e, 0x8000000000000000};
#define EventD3D12AutoBreadcrumbEntry_value 0x91
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateTrackedWorkload = {0x92, 0x0, 0x10, 0x0, 0x1, 0x3f, 0x8000000000000084};
#define EventD3D12CreateTrackedWorkload_value 0x92
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyTrackedWorkload = {0x93, 0x0, 0x10, 0x0, 0x2, 0x3f, 0x8000000000000084};
#define EventD3D12DestroyTrackedWorkload_value 0x93
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportTrackedWorkload = {0x94, 0x0, 0x10, 0x0, 0x3, 0x3f, 0x8000000000000084};
#define EventD3D12ReportTrackedWorkload_value 0x94
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateVideoEncoder = {0x95, 0x0, 0x10, 0x0, 0x1, 0x40, 0x8000000000000084};
#define EventD3D12CreateVideoEncoder_value 0x95
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyVideoEncoder = {0x96, 0x0, 0x10, 0x0, 0x2, 0x40, 0x8000000000000084};
#define EventD3D12DestroyVideoEncoder_value 0x96
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportVideoEncoder = {0x97, 0x0, 0x10, 0x0, 0x3, 0x40, 0x8000000000000084};
#define EventD3D12ReportVideoEncoder_value 0x97
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateVideoEncoderHeap = {0x98, 0x0, 0x10, 0x0, 0x1, 0x41, 0x8000000000000084};
#define EventD3D12CreateVideoEncoderHeap_value 0x98
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12DestroyVideoEncoderHeap = {0x99, 0x0, 0x10, 0x0, 0x2, 0x41, 0x8000000000000084};
#define EventD3D12DestroyVideoEncoderHeap_value 0x99
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12ReportVideoEncoderHeap = {0x9a, 0x0, 0x10, 0x0, 0x3, 0x41, 0x8000000000000084};
#define EventD3D12ReportVideoEncoderHeap_value 0x9a
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12BeginCreatePipelineStateObject = {0x9b, 0x0, 0x12, 0x4, 0x1, 0x42, 0x2000000000000400};
#define EventD3D12BeginCreatePipelineStateObject_value 0x9b
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12EndCreatePipelineStateObject = {0x9c, 0x0, 0x12, 0x4, 0x2, 0x42, 0x2000000000000400};
#define EventD3D12EndCreatePipelineStateObject_value 0x9c
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12BeginCreateStateObject = {0x9d, 0x0, 0x12, 0x4, 0x1, 0x43, 0x2000000000000400};
#define EventD3D12BeginCreateStateObject_value 0x9d
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12EndCreateStateObject = {0x9e, 0x0, 0x12, 0x4, 0x2, 0x43, 0x2000000000000400};
#define EventD3D12EndCreateStateObject_value 0x9e
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12BeginAddToStateObject = {0x9f, 0x0, 0x12, 0x4, 0x1, 0x44, 0x2000000000000400};
#define EventD3D12BeginAddToStateObject_value 0x9f
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12EndAddToStateObject = {0xa0, 0x0, 0x12, 0x4, 0x2, 0x44, 0x2000000000000400};
#define EventD3D12EndAddToStateObject_value 0xa0
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreatePipelineStateObjectCacheStatistics = {0xa1, 0x0, 0x12, 0x4, 0x0, 0x42, 0x2000000000000400};
#define EventD3D12CreatePipelineStateObjectCacheStatistics_value 0xa1
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12CreateStateObjectCacheStatistics = {0xa2, 0x0, 0x12, 0x4, 0x0, 0x43, 0x2000000000000400};
#define EventD3D12CreateStateObjectCacheStatistics_value 0xa2
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12AddToStateObjectCacheStatistics = {0xa3, 0x0, 0x12, 0x4, 0x0, 0x44, 0x2000000000000400};
#define EventD3D12AddToStateObjectCacheStatistics_value 0xa3
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12AddPipelineStateToSODB = {0xa4, 0x1, 0x10, 0x4, 0x0, 0x45, 0x8000000000000800};
#define EventD3D12AddPipelineStateToSODB_value 0xa4
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12AddStateObjectToSODB = {0xa5, 0x1, 0x10, 0x4, 0x0, 0x45, 0x8000000000000800};
#define EventD3D12AddStateObjectToSODB_value 0xa5
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12AddSubObjectToSODB = {0xa6, 0x1, 0x10, 0x5, 0x0, 0x45, 0x8000000000000800};
#define EventD3D12AddSubObjectToSODB_value 0xa6
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12BeginVideoEncodeFrame = {0xa7, 0x0, 0x10, 0x0, 0x1, 0x46, 0x8000000000002000};
#define EventD3D12BeginVideoEncodeFrame_value 0xa7
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12EndVideoEncodeFrame = {0xa8, 0x0, 0x10, 0x0, 0x2, 0x46, 0x8000000000002000};
#define EventD3D12EndVideoEncodeFrame_value 0xa8
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12BeginVideoDecodeFrame = {0xa9, 0x0, 0x10, 0x0, 0x1, 0x47, 0x8000000000002000};
#define EventD3D12BeginVideoDecodeFrame_value 0xa9
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12EndVideoDecodeFrame = {0xaa, 0x0, 0x10, 0x0, 0x2, 0x47, 0x8000000000002000};
#define EventD3D12EndVideoDecodeFrame_value 0xaa
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12BeginVideoProcessFrame = {0xab, 0x0, 0x10, 0x0, 0x1, 0x48, 0x8000000000002000};
#define EventD3D12BeginVideoProcessFrame_value 0xab
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12EndVideoProcessFrame = {0xac, 0x0, 0x10, 0x0, 0x2, 0x48, 0x8000000000002000};
#define EventD3D12EndVideoProcessFrame_value 0xac
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12BeginVideoResolveEncoderOutputMetadata = {0xad, 0x0, 0x10, 0x0, 0x1, 0x46, 0x8000000000002000};
#define EventD3D12BeginVideoResolveEncoderOutputMetadata_value 0xad
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12EndVideoResolveEncoderOutputMetadata = {0xae, 0x0, 0x10, 0x0, 0x2, 0x46, 0x8000000000002000};
#define EventD3D12EndVideoResolveEncoderOutputMetadata_value 0xae
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12BeginVideoResolveInputParamLayout = {0xaf, 0x0, 0x10, 0x0, 0x1, 0x46, 0x8000000000002000};
#define EventD3D12BeginVideoResolveInputParamLayout_value 0xaf
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12EndVideoResolveInputParamLayout = {0xb0, 0x0, 0x10, 0x0, 0x2, 0x46, 0x8000000000002000};
#define EventD3D12EndVideoResolveInputParamLayout_value 0xb0
EXTERN_C __declspec(selectany) const EVENT_DESCRIPTOR EventD3D12VideoEncodeEngineAccessDenied = {0xb1, 0x0, 0x10, 0x0, 0x0, 0x2, 0x8000000000000002};
#define EventD3D12VideoEncodeEngineAccessDenied_value 0xb1

//
// MCGEN_DISABLE_PROVIDER_CODE_GENERATION macro:
// Define this macro to have the compiler skip the generated functions in this
// header.
//

//
// MCGEN_DISABLE_PROVIDER_CODE_GENERATION macro:
// Define this macro to have the compiler skip the generated functions in this
// header.
//

#if defined(__cplusplus)
}
#endif

#define MSG_Map_D3D12_D3D_FEATURE_LEVEL_9_1  0xD0000001L
#define MSG_Map_D3D12_D3D_FEATURE_LEVEL_9_2  0xD0000002L
#define MSG_Map_D3D12_D3D_FEATURE_LEVEL_9_3  0xD0000003L
#define MSG_Map_D3D12_D3D_FEATURE_LEVEL_10_0 0xD0000004L
#define MSG_Map_D3D12_D3D_FEATURE_LEVEL_10_1 0xD0000005L
#define MSG_Map_D3D12_D3D_FEATURE_LEVEL_11_0 0xD0000006L
#define MSG_Map_D3D12DDI_FEATURE_0020_UNKNOWN 0xD0000007L
#define MSG_Map_D3D12DDI_FEATURE_0020_CORE   0xD0000008L
#define MSG_Map_D3D12DDI_FEATURE_0020_VIDEO  0xD0000009L
#define MSG_Map_D3D12DDI_FEATURE_0021_SHADERCACHING 0xD000000AL
#define MSG_Map_D3D12DDI_FEATURE_0030_CONTENT_PROTECTION_RESOURCES 0xD000000BL
#define MSG_Map_D3D12DDI_FEATURE_0030_CONTENT_PROTECTION_STREAMING 0xD000000CL
#define MSG_Map_D3D12DDI_FEATURE_0033_METACOMMAND 0xD000000DL
#define MSG_Map_D3D12DDI_FEATURE_0043_RENDER_PASS 0xD000000EL
#define MSG_Map_D3D12_RESOURCE_DIMENSION_UNKNOWN 0xD000000FL
#define MSG_Map_D3D12_RESOURCE_DIMENSION_BUFFER 0xD0000010L
#define MSG_Map_D3D12_RESOURCE_DIMENSION_TEXTURE1D 0xD0000011L
#define MSG_Map_D3D12_RESOURCE_DIMENSION_TEXTURE2D 0xD0000012L
#define MSG_Map_D3D12_RESOURCE_DIMENSION_TEXTURE3D 0xD0000013L
#define MSG_Map_D3D12_TEXTURE_LAYOUT_UNKNOWN 0xD0000014L
#define MSG_Map_D3D12_TEXTURE_LAYOUT_ROW_MAJOR 0xD0000015L
#define MSG_Map_D3D12_TEXTURE_LAYOUT_64KB_UNDEFINED_SWIZZLE 0xD0000016L
#define MSG_Map_D3D12_TEXTURE_LAYOUT_64KB_STANDARD_SWIZZLE 0xD0000017L
#define MSG_Map_D3D12_INTERNAL_RESOURCE_HEAP_TYPE_IMMUTABLE_HEAP 0xD0000018L
#define MSG_Map_D3D12_INTERNAL_RESOURCE_HEAP_TYPE_IMPLICIT_HEAP 0xD0000019L
#define MSG_Map_D3D12_INTERNAL_RESOURCE_HEAP_TYPE_IMPLICIT_RESOURCE 0xD000001AL
#define MSG_Map_D3D12_INTERNAL_RESOURCE_HEAP_TYPE_RESERVED_RESOURCE 0xD000001BL
#define MSG_Map_D3D12_INTERNAL_RESOURCE_HEAP_TYPE_IMMUTABLE_RESOURCE 0xD000001CL
#define MSG_Map_D3D12_DEVICE_SINGLE_ADAPTER_HYBRID_MODE_UNSPECIFIED 0xD000001DL
#define MSG_Map_D3D12_DEVICE_SINGLE_ADAPTER_HYBRID_MODE_MINIMUM_POWER 0xD000001EL
#define MSG_Map_D3D12_DEVICE_SINGLE_ADAPTER_HYBRID_MODE_HIGH_PERFORMANCE 0xD000001FL
#define MSG_Map_D3D12_DXGI_FORMAT_UNKNOWN    0xD0000020L
#define MSG_Map_D3D12_DXGI_FORMAT_R32G32B32A32_TYPELESS 0xD0000021L
#define MSG_Map_D3D12_DXGI_FORMAT_R32G32B32A32_FLOAT 0xD0000022L
#define MSG_Map_D3D12_DXGI_FORMAT_R32G32B32A32_UINT 0xD0000023L
#define MSG_Map_D3D12_DXGI_FORMAT_R32G32B32A32_SINT 0xD0000024L
#define MSG_Map_D3D12_DXGI_FORMAT_R32G32B32_TYPELESS 0xD0000025L
#define MSG_Map_D3D12_DXGI_FORMAT_R32G32B32_FLOAT 0xD0000026L
#define MSG_Map_D3D12_DXGI_FORMAT_R32G32B32_UINT 0xD0000027L
#define MSG_Map_D3D12_DXGI_FORMAT_R32G32B32_SINT 0xD0000028L
#define MSG_Map_D3D12_DXGI_FORMAT_R16G16B16A16_TYPELESS 0xD0000029L
#define MSG_Map_D3D12_DXGI_FORMAT_R16G16B16A16_FLOAT 0xD000002AL
#define MSG_Map_D3D12_DXGI_FORMAT_R16G16B16A16_UNORM 0xD000002BL
#define MSG_Map_D3D12_DXGI_FORMAT_R16G16B16A16_UINT 0xD000002CL
#define MSG_Map_D3D12_DXGI_FORMAT_R16G16B16A16_SNORM 0xD000002DL
#define MSG_Map_D3D12_DXGI_FORMAT_R16G16B16A16_SINT 0xD000002EL
#define MSG_Map_D3D12_DXGI_FORMAT_R32G32_TYPELESS 0xD000002FL
#define MSG_Map_D3D12_DXGI_FORMAT_R32G32_FLOAT 0xD0000030L
#define MSG_Map_D3D12_DXGI_FORMAT_R32G32_UINT 0xD0000031L
#define MSG_Map_D3D12_DXGI_FORMAT_R32G32_SINT 0xD0000032L
#define MSG_Map_D3D12_DXGI_FORMAT_R32G8X24_TYPELESS 0xD0000033L
#define MSG_Map_D3D12_DXGI_FORMAT_D32_FLOAT_S8X24_UINT 0xD0000034L
#define MSG_Map_D3D12_DXGI_FORMAT_R32_FLOAT_X8X24_TYPELESS 0xD0000035L
#define MSG_Map_D3D12_DXGI_FORMAT_X32_TYPELESS_G8X24_UINT 0xD0000036L
#define MSG_Map_D3D12_DXGI_FORMAT_R10G10B10A2_TYPELESS 0xD0000037L
#define MSG_Map_D3D12_DXGI_FORMAT_R10G10B10A2_UNORM 0xD0000038L
#define MSG_Map_D3D12_DXGI_FORMAT_R10G10B10A2_UINT 0xD0000039L
#define MSG_Map_D3D12_DXGI_FORMAT_R11G11B10_FLOAT 0xD000003AL
#define MSG_Map_D3D12_DXGI_FORMAT_R8G8B8A8_TYPELESS 0xD000003BL
#define MSG_Map_D3D12_DXGI_FORMAT_R8G8B8A8_UNORM 0xD000003CL
#define MSG_Map_D3D12_DXGI_FORMAT_R8G8B8A8_UNORM_SRGB 0xD000003DL
#define MSG_Map_D3D12_DXGI_FORMAT_R8G8B8A8_UINT 0xD000003EL
#define MSG_Map_D3D12_DXGI_FORMAT_R8G8B8A8_SNORM 0xD000003FL
#define MSG_Map_D3D12_DXGI_FORMAT_R8G8B8A8_SINT 0xD0000040L
#define MSG_Map_D3D12_DXGI_FORMAT_R16G16_TYPELESS 0xD0000041L
#define MSG_Map_D3D12_DXGI_FORMAT_R16G16_FLOAT 0xD0000042L
#define MSG_Map_D3D12_DXGI_FORMAT_R16G16_UNORM 0xD0000043L
#define MSG_Map_D3D12_DXGI_FORMAT_R16G16_UINT 0xD0000044L
#define MSG_Map_D3D12_DXGI_FORMAT_R16G16_SNORM 0xD0000045L
#define MSG_Map_D3D12_DXGI_FORMAT_R16G16_SINT 0xD0000046L
#define MSG_Map_D3D12_DXGI_FORMAT_R32_TYPELESS 0xD0000047L
#define MSG_Map_D3D12_DXGI_FORMAT_D32_FLOAT  0xD0000048L
#define MSG_Map_D3D12_DXGI_FORMAT_R32_FLOAT  0xD0000049L
#define MSG_Map_D3D12_DXGI_FORMAT_R32_UINT   0xD000004AL
#define MSG_Map_D3D12_DXGI_FORMAT_R32_SINT   0xD000004BL
#define MSG_Map_D3D12_DXGI_FORMAT_R24G8_TYPELESS 0xD000004CL
#define MSG_Map_D3D12_DXGI_FORMAT_D24_UNORM_S8_UINT 0xD000004DL
#define MSG_Map_D3D12_DXGI_FORMAT_R24_UNORM_X8_TYPELESS 0xD000004EL
#define MSG_Map_D3D12_DXGI_FORMAT_X24_TYPELESS_G8_UINT 0xD000004FL
#define MSG_Map_D3D12_DXGI_FORMAT_R8G8_TYPELESS 0xD0000050L
#define MSG_Map_D3D12_DXGI_FORMAT_R8G8_UNORM 0xD0000051L
#define MSG_Map_D3D12_DXGI_FORMAT_R8G8_UINT  0xD0000052L
#define MSG_Map_D3D12_DXGI_FORMAT_R8G8_SNORM 0xD0000053L
#define MSG_Map_D3D12_DXGI_FORMAT_R8G8_SINT  0xD0000054L
#define MSG_Map_D3D12_DXGI_FORMAT_R16_TYPELESS 0xD0000055L
#define MSG_Map_D3D12_DXGI_FORMAT_R16_FLOAT  0xD0000056L
#define MSG_Map_D3D12_DXGI_FORMAT_D16_UNORM  0xD0000057L
#define MSG_Map_D3D12_DXGI_FORMAT_R16_UNORM  0xD0000058L
#define MSG_Map_D3D12_DXGI_FORMAT_R16_UINT   0xD0000059L
#define MSG_Map_D3D12_DXGI_FORMAT_R16_SNORM  0xD000005AL
#define MSG_Map_D3D12_DXGI_FORMAT_R16_SINT   0xD000005BL
#define MSG_Map_D3D12_DXGI_FORMAT_R8_TYPELESS 0xD000005CL
#define MSG_Map_D3D12_DXGI_FORMAT_R8_UNORM   0xD000005DL
#define MSG_Map_D3D12_DXGI_FORMAT_R8_UINT    0xD000005EL
#define MSG_Map_D3D12_DXGI_FORMAT_R8_SNORM   0xD000005FL
#define MSG_Map_D3D12_DXGI_FORMAT_R8_SINT    0xD0000060L
#define MSG_Map_D3D12_DXGI_FORMAT_A8_UNORM   0xD0000061L
#define MSG_Map_D3D12_DXGI_FORMAT_R1_UNORM   0xD0000062L
#define MSG_Map_D3D12_DXGI_FORMAT_R9G9B9E5_SHAREDEXP 0xD0000063L
#define MSG_Map_D3D12_DXGI_FORMAT_R8G8_B8G8_UNORM 0xD0000064L
#define MSG_Map_D3D12_DXGI_FORMAT_G8R8_G8B8_UNORM 0xD0000065L
#define MSG_Map_D3D12_DXGI_FORMAT_BC1_TYPELESS 0xD0000066L
#define MSG_Map_D3D12_DXGI_FORMAT_BC1_UNORM  0xD0000067L
#define MSG_Map_D3D12_DXGI_FORMAT_BC1_UNORM_SRGB 0xD0000068L
#define MSG_Map_D3D12_DXGI_FORMAT_BC2_TYPELESS 0xD0000069L
#define MSG_Map_D3D12_DXGI_FORMAT_BC2_UNORM  0xD000006AL
#define MSG_Map_D3D12_DXGI_FORMAT_BC2_UNORM_SRGB 0xD000006BL
#define MSG_Map_D3D12_DXGI_FORMAT_BC3_TYPELESS 0xD000006CL
#define MSG_Map_D3D12_DXGI_FORMAT_BC3_UNORM  0xD000006DL
#define MSG_Map_D3D12_DXGI_FORMAT_BC3_UNORM_SRGB 0xD000006EL
#define MSG_Map_D3D12_DXGI_FORMAT_BC4_TYPELESS 0xD000006FL
#define MSG_Map_D3D12_DXGI_FORMAT_BC4_UNORM  0xD0000070L
#define MSG_Map_D3D12_DXGI_FORMAT_BC4_SNORM  0xD0000071L
#define MSG_Map_D3D12_DXGI_FORMAT_BC5_TYPELESS 0xD0000072L
#define MSG_Map_D3D12_DXGI_FORMAT_BC5_UNORM  0xD0000073L
#define MSG_Map_D3D12_DXGI_FORMAT_BC5_SNORM  0xD0000074L
#define MSG_Map_D3D12_DXGI_FORMAT_B5G6R5_UNORM 0xD0000075L
#define MSG_Map_D3D12_DXGI_FORMAT_B5G5R5A1_UNORM 0xD0000076L
#define MSG_Map_D3D12_DXGI_FORMAT_B8G8R8A8_UNORM 0xD0000077L
#define MSG_Map_D3D12_DXGI_FORMAT_B8G8R8X8_UNORM 0xD0000078L
#define MSG_Map_D3D12_DXGI_FORMAT_R10G10B10_XR_BIAS_A2_UNORM 0xD0000079L
#define MSG_Map_D3D12_DXGI_FORMAT_B8G8R8A8_TYPELESS 0xD000007AL
#define MSG_Map_D3D12_DXGI_FORMAT_B8G8R8A8_UNORM_SRGB 0xD000007BL
#define MSG_Map_D3D12_DXGI_FORMAT_B8G8R8X8_TYPELESS 0xD000007CL
#define MSG_Map_D3D12_DXGI_FORMAT_B8G8R8X8_UNORM_SRGB 0xD000007DL
#define MSG_Map_D3D12_DXGI_FORMAT_BC6H_TYPELESS 0xD000007EL
#define MSG_Map_D3D12_DXGI_FORMAT_BC6H_UF16  0xD000007FL
#define MSG_Map_D3D12_DXGI_FORMAT_BC6H_SF16  0xD0000080L
#define MSG_Map_D3D12_DXGI_FORMAT_BC7_TYPELESS 0xD0000081L
#define MSG_Map_D3D12_DXGI_FORMAT_BC7_UNORM  0xD0000082L
#define MSG_Map_D3D12_DXGI_FORMAT_BC7_UNORM_SRGB 0xD0000083L
#define MSG_Map_D3D12_DXGI_FORMAT_AYUV       0xD0000084L
#define MSG_Map_D3D12_DXGI_FORMAT_Y410       0xD0000085L
#define MSG_Map_D3D12_DXGI_FORMAT_Y416       0xD0000086L
#define MSG_Map_D3D12_DXGI_FORMAT_NV12       0xD0000087L
#define MSG_Map_D3D12_DXGI_FORMAT_P010       0xD0000088L
#define MSG_Map_D3D12_DXGI_FORMAT_P016       0xD0000089L
#define MSG_Map_D3D12_DXGI_FORMAT_420_OPAQUE 0xD000008AL
#define MSG_Map_D3D12_DXGI_FORMAT_YUY2       0xD000008BL
#define MSG_Map_D3D12_DXGI_FORMAT_Y210       0xD000008CL
#define MSG_Map_D3D12_DXGI_FORMAT_Y216       0xD000008DL
#define MSG_Map_D3D12_DXGI_FORMAT_NV11       0xD000008EL
#define MSG_Map_D3D12_DXGI_FORMAT_AI44       0xD000008FL
#define MSG_Map_D3D12_DXGI_FORMAT_IA44       0xD0000090L
#define MSG_Map_D3D12_DXGI_FORMAT_P8         0xD0000091L
#define MSG_Map_D3D12_DXGI_FORMAT_A8P8       0xD0000092L
#define MSG_Map_D3D12_DXGI_FORMAT_B4G4R4A4_UNORM 0xD0000093L
#define MSG_Map_D3D12_DXGI_FORMAT_R10G10B10_7E3_A2_FLOAT 0xD0000094L
#define MSG_Map_D3D12_DXGI_FORMAT_R10G10B10_6E4_A2_FLOAT 0xD0000095L
#define MSG_Map_D3D12_DXGI_FORMAT_D16_UNORM_S8_UINT 0xD0000096L
#define MSG_Map_D3D12_DXGI_FORMAT_R16_UNORM_X8_TYPELESS 0xD0000097L
#define MSG_Map_D3D12_DXGI_FORMAT_X16_TYPELESS_G8_UINT 0xD0000098L
#define MSG_Map_D3D12_DXGI_FORMAT_B5G5R5X1_UNORM 0xD0000099L
#define MSG_Map_D3D12_DXGI_FORMAT_B4G4R4X4_UNORM 0xD000009AL
#define MSG_Map_D3D12_DXGI_FORMAT_L8_UNORM   0xD000009BL
#define MSG_Map_D3D12_DXGI_FORMAT_L16_UNORM  0xD000009CL
#define MSG_Map_D3D12_DXGI_FORMAT_L8A8_UNORM 0xD000009DL
#define MSG_Map_D3D12_DXGI_FORMAT_R10G10B10A2_UNORM_SRGB 0xD000009EL
#define MSG_Map_D3D12_DXGI_FORMAT_R10G10B10X2_UINT 0xD000009FL
#define MSG_Map_D3D12_DXGI_FORMAT_R10G10B10X2_SNORM 0xD00000A0L
#define MSG_Map_D3D12_DXGI_FORMAT_B10G10R10A2_UNORM 0xD00000A1L
#define MSG_Map_D3D12_DXGI_FORMAT_P208       0xD00000A2L
#define MSG_Map_D3D12_DXGI_FORMAT_V208       0xD00000A3L
#define MSG_Map_D3D12_DXGI_FORMAT_V408       0xD00000A4L
#define MSG_Map_D3D12_MARKER_API_SETMARKER   0xD00000A5L
#define MSG_Map_D3D12_MARKER_API_BEGINEVENT  0xD00000A6L
#define MSG_Map_D3D12_MARKER_API_ENDEVENT    0xD00000A7L
#define MSG_Map_D3D12_MARKER_API_DRAWINSTANCED 0xD00000A8L
#define MSG_Map_D3D12_MARKER_API_DRAWINDEXEDINSTANCED 0xD00000A9L
#define MSG_Map_D3D12_MARKER_API_EXECUTEINDIRECT 0xD00000AAL
#define MSG_Map_D3D12_MARKER_API_DISPATCH    0xD00000ABL
#define MSG_Map_D3D12_MARKER_API_COPYBUFFERREGION 0xD00000ACL
#define MSG_Map_D3D12_MARKER_API_COPYTEXTUREREGION 0xD00000ADL
#define MSG_Map_D3D12_MARKER_API_COPYRESOURCE 0xD00000AEL
#define MSG_Map_D3D12_MARKER_API_COPYTILES   0xD00000AFL
#define MSG_Map_D3D12_MARKER_API_RESOLVESUBRESOURCE 0xD00000B0L
#define MSG_Map_D3D12_MARKER_API_CLEARRENDERTARGETVIEW 0xD00000B1L
#define MSG_Map_D3D12_MARKER_API_CLEARUNORDEREDACCESSVIEW 0xD00000B2L
#define MSG_Map_D3D12_MARKER_API_CLEARDEPTHSTENCILVIEW 0xD00000B3L
#define MSG_Map_D3D12_MARKER_API_RESOURCEBARRIER 0xD00000B4L
#define MSG_Map_D3D12_MARKER_API_EXECUTEBUNDLE 0xD00000B5L
#define MSG_Map_D3D12_MARKER_API_PRESENT     0xD00000B6L
#define MSG_Map_D3D12_MARKER_API_RESOLVEQUERYDATA 0xD00000B7L
#define MSG_Map_D3D12_MARKER_API_BEGINSUBMISSION 0xD00000B8L
#define MSG_Map_D3D12_MARKER_API_ENDSUBMISSION 0xD00000B9L
#define MSG_Map_D3D12_MARKER_API_DECODEFRAME 0xD00000BAL
#define MSG_Map_D3D12_MARKER_API_PROCESSFRAMES 0xD00000BBL
#define MSG_Map_D3D12_MARKER_API_ATOMICCOPYBUFFERUINT 0xD00000BCL
#define MSG_Map_D3D12_MARKER_API_ATOMICCOPYBUFFERUINT64 0xD00000BDL
#define MSG_Map_D3D12_MARKER_API_RESOLVESUBRESOURCEREGION 0xD00000BEL
#define MSG_Map_D3D12_MARKER_API_WRITEBUFFERIMMEDIATE 0xD00000BFL
#define MSG_Map_D3D12_MARKER_API_DECODEFRAME1 0xD00000C0L
#define MSG_Map_D3D12_MARKER_API_SETPROTECTEDRESOURCESESSION 0xD00000C1L
#define MSG_Map_D3D12_MARKER_API_DECODEFRAME2 0xD00000C2L
#define MSG_Map_D3D12_MARKER_API_PROCESSFRAMES1 0xD00000C3L
#define MSG_Map_D3D12_MARKER_API_BUILDRAYTRACINGACCELERATIONSTRUCTURE 0xD00000C4L
#define MSG_Map_D3D12_MARKER_API_EMITRAYTRACINGACCELERATIONSTRUCTUREPOSTBUILDINFO 0xD00000C5L
#define MSG_Map_D3D12_MARKER_API_COPYRAYTRACINGACCELERATIONSTRUCTURE 0xD00000C6L
#define MSG_Map_D3D12_MARKER_API_DISPATCHRAYS 0xD00000C7L
#define MSG_Map_D3D12_MARKER_API_INITIALIZEMETACOMMAND 0xD00000C8L
#define MSG_Map_D3D12_MARKER_API_EXECUTEMETACOMMAND 0xD00000C9L
#define MSG_Map_D3D12_MARKER_API_ESTIMATEMOTION 0xD00000CAL
#define MSG_Map_D3D12_MARKER_API_RESOLVEMOTIONVECTORHEAP 0xD00000CBL
#define MSG_Map_D3D12_MARKER_API_SETPIPELINESTATE1 0xD00000CCL
#define MSG_Map_D3D12_MARKER_API_INITIALIZEEXTENSIONCOMMAND 0xD00000CDL
#define MSG_Map_D3D12_MARKER_API_EXECUTEEXTENSIONCOMMAND 0xD00000CEL
#define MSG_Map_D3D12_MARKER_API_DISPATCHMESH 0xD00000CFL
#define MSG_Map_D3D12_MARKER_API_ENCODEFRAME 0xD00000D0L
#define MSG_Map_D3D12_MARKER_API_RESOLVEENCODEROUTPUTMETADATA 0xD00000D1L
#define MSG_Map_D3D12_MARKER_API_BARRIER     0xD00000D2L
#define MSG_Map_D3D12_MARKER_API_BEGIN_COMMAND_LIST 0xD00000D3L
#define MSG_Map_D3D12_MARKER_API_DISPATCHGRAPH 0xD00000D4L
#define MSG_Map_D3D12_MARKER_API_SETPROGRAM  0xD00000D5L
#define MSG_Map_D3D12_MARKER_API_ENCODEFRAME1 0xD00000D6L
#define MSG_Map_D3D12_MARKER_API_RESOLVEENCODEROUTPUTMETADATA1 0xD00000D7L
#define MSG_Map_D3D12_MARKER_API_RESOLVEINPUTPARAMLAYOUT 0xD00000D8L
#define MSG_Map_D3D12_MARKER_API_PROCESSFRAMES2 0xD00000D9L
#define MSG_Map_D3D12_MARKER_API_SET_WORK_GRAPH_MAXIMUM_GPU_INPUT_RECORDS 0xD00000DAL
#define MSG_Map_D3D12_MARKER_LOG_TYPE_NONE   0xD00000DBL
#define MSG_Map_D3D12_MARKER_LOG_TYPE_PROFILE 0xD00000DCL
#define MSG_Map_D3D12_COMMAND_LIST_DIRECT    0xD00000DDL
#define MSG_Map_D3D12_COMMAND_LIST_BUNDLE    0xD00000DEL
#define MSG_Map_D3D12_COMMAND_LIST_COMPUTE   0xD00000DFL
#define MSG_Map_D3D12_COMMAND_LIST_COPY      0xD00000E0L
#define MSG_Map_D3D12_COMMAND_LIST_VIDEO_DECODE 0xD00000E1L
#define MSG_Map_D3D12_COMMAND_LIST_VIDEO_PROCESS 0xD00000E2L
#define MSG_Map_D3D12_COMMAND_LIST_VIDEO_ENCODE 0xD00000E3L
#define MSG_Map_D3D12_HEAP_TYPE_UNKNOWN      0xD00000E4L
#define MSG_Map_D3D12_HEAP_TYPE_DEFAULT      0xD00000E5L
#define MSG_Map_D3D12_HEAP_TYPE_UPLOAD       0xD00000E6L
#define MSG_Map_D3D12_HEAP_TYPE_READBACK     0xD00000E7L
#define MSG_Map_D3D12_HEAP_TYPE_CUSTOM       0xD00000E8L
#define MSG_Map_D3D12_CPU_PAGE_PROPERTY_UNKNOWN 0xD00000E9L
#define MSG_Map_D3D12_CPU_PAGE_PROPERTY_NOT_AVAILABLE 0xD00000EAL
#define MSG_Map_D3D12_CPU_PAGE_PROPERTY_WRITE_COMBINE 0xD00000EBL
#define MSG_Map_D3D12_CPU_PAGE_PROPERTY_WRITE_BACK 0xD00000ECL
#define MSG_Map_D3D12_MEMORY_POOL_UNKNOWN    0xD00000EDL
#define MSG_Map_D3D12_MEMORY_POOL_L0         0xD00000EEL
#define MSG_Map_D3D12_MEMORY_POOL_L1         0xD00000EFL
#define MSG_Map_D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV 0xD00000F0L
#define MSG_Map_D3D12_DESCRIPTOR_HEAP_TYPE_SAMPLER 0xD00000F1L
#define MSG_Map_D3D12_DESCRIPTOR_HEAP_TYPE_RTV 0xD00000F2L
#define MSG_Map_D3D12_DESCRIPTOR_HEAP_TYPE_DSV 0xD00000F3L
#define MSG_Map_D3D12_DESCRIPTOR_HEAP_TYPE_NUM_TYPES 0xD00000F4L
#define MSG_Map_D3D12_QUERY_HEAP_TYPE_OCCLUSION 0xD00000F5L
#define MSG_Map_D3D12_QUERY_HEAP_TYPE_TIMESTAMP 0xD00000F6L
#define MSG_Map_D3D12_QUERY_HEAP_TYPE_PIPELINE_STATISTICS 0xD00000F7L
#define MSG_Map_D3D12_QUERY_HEAP_TYPE_SO_STATISTICS 0xD00000F8L
#define MSG_Map_D3D12_QUERY_HEAP_TYPE_VIDEO_DECODE_STATISTICS 0xD00000F9L
#define MSG_Map_D3D12_QUERY_HEAP_TYPE_JPEG   0xD00000FAL
#define MSG_Map_D3D12_QUERY_HEAP_TYPE_PIPELINE_STATISTICS1 0xD00000FBL
#define MSG_Map_D3D12_VIDEO_FRAME_CODED_INTERLACE_TYPE_NONE 0xD00000FCL
#define MSG_Map_D3D12_VIDEO_FRAME_CODED_INTERLACE_TYPE_FIELD_BASED 0xD00000FDL
#define MSG_Map_D3D12_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE_8X8 0xD00000FEL
#define MSG_Map_D3D12_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE_16X16 0xD00000FFL
#define MSG_Map_D3D12_MOTION_ESTIMATOR_VECTOR_PRECISION_QUARTER_PEL 0xD0000100L
#define MSG_Map_D3D12_SHADER_CACHE_MODE_MEMORY 0xD0000101L
#define MSG_Map_D3D12_SHADER_CACHE_MODE_DISK 0xD0000102L
#define MSG_Map_D3D12_VIDEO_ENCODER_FLAG_NONE 0xD0000103L
#define MSG_Map_D3D12_VIDEO_ENCODER_CODEC_H264 0xD0000104L
#define MSG_Map_D3D12_VIDEO_ENCODER_CODEC_HEVC 0xD0000105L
#define MSG_Map_D3D12_VIDEO_ENCODER_MOTION_ESTIMATION_PRECISION_MODE_MAXIMUM 0xD0000106L
#define MSG_Map_D3D12_VIDEO_ENCODER_MOTION_ESTIMATION_PRECISION_MODE_FULL_PIXEL 0xD0000107L
#define MSG_Map_D3D12_VIDEO_ENCODER_MOTION_ESTIMATION_PRECISION_MODE_HALF_PIXEL 0xD0000108L
#define MSG_Map_D3D12_VIDEO_ENCODER_MOTION_ESTIMATION_PRECISION_MODE_QUARTER_PIXEL 0xD0000109L
#define MSG_Map_D3D12_VIDEO_ENCODER_HEAP_FLAG_NONE 0xD000010AL
#define MSG_Map_D3D12_FEATURE_THREADING_DRIVER_CONCURRENT_CREATES 0xF0000001L
#define MSG_Map_D3D12_FEATURE_DOUBLES_DOUBLE_PRECISION_FLOAT_SHADER_OPS 0xF0000002L
#define MSG_Map_D3D12_FEATURE_D3D10_X_HARDWARE_OPTIONS_COMPUTE_SHADERS_PLUS_RAW_AND_STRUCTURED_BUFFERS_VIA_SHADER_4_X 0xF0000003L
#define MSG_Map_D3D12_D3D10DDI_CREATEDEVICE_FLAG_DISABLE_EXTRA_THREAD_CREATION 0xF0000004L
#define MSG_Map_D3D12_RESOURCE_FLAG_ALLOW_RENDER_TARGET 0xF0000005L
#define MSG_Map_D3D12_RESOURCE_FLAG_ALLOW_DEPTH_STENCIL 0xF0000006L
#define MSG_Map_D3D12_RESOURCE_FLAG_ALLOW_UNORDERED_ACCESS 0xF0000007L
#define MSG_Map_D3D12_RESOURCE_FLAG_DENY_SHADER_RESOURCE 0xF0000008L
#define MSG_Map_D3D12_RESOURCE_FLAG_ALLOW_CROSS_ADAPTER 0xF0000009L
#define MSG_Map_D3D12_RESOURCE_FLAG_ALLOW_SIMULTANEOUS_ACCESS 0xF000000AL
#define MSG_Map_D3D12_RESOURCE_FLAG_VIDEO_DECODE_REFERENCE_ONLY 0xF000000BL
#define MSG_Map_D3D12_RESOURCE_FLAG_VIDEO_ENCODE_REFERENCE_ONLY 0xF000000CL
#define MSG_Map_D3D12_RESOURCE_FLAG_VIDEO_PROCESS_3DLUT_ONLY 0xF000000DL
#define MSG_Map_D3D12_COMMAND_QUEUE_FLAG_DISABLE_GPU_TIMEOUT 0xF000000EL
#define MSG_Map_D3D12_HEAP_FLAG_SHARED       0xF000000FL
#define MSG_Map_D3D12_HEAP_FLAG_DENY_BUFFERS 0xF0000010L
#define MSG_Map_D3D12_HEAP_FLAG_ALLOW_DISPLAY 0xF0000011L
#define MSG_Map_D3D12_HEAP_FLAG_SHARED_CROSS_ADAPTER 0xF0000012L
#define MSG_Map_D3D12_HEAP_FLAG_DENY_RT_DS_TEXTURES 0xF0000013L
#define MSG_Map_D3D12_HEAP_FLAG_DENY_NON_RT_DS_TEXTURES 0xF0000014L
#define MSG_Map_D3D12_HEAP_FLAG_HARDWARE_PROTECTED 0xF0000015L
#define MSG_Map_D3D12_HEAP_FLAG_ALLOW_WRITE_WATCH 0xF0000016L
#define MSG_Map_D3D12_HEAP_FLAG_ALLOW_SHADER_ATOMICS 0xF0000017L
#define MSG_Map_D3D12_DESCRIPTOR_HEAP_FLAG_SHADER_VISIBLE 0xF0000018L
#define MSG_Map_D3D12_COMMAND_LIST_SUPPORT_FLAG_DIRECT 0xF0000019L
#define MSG_Map_D3D12_COMMAND_LIST_SUPPORT_FLAG_BUNDLE 0xF000001AL
#define MSG_Map_D3D12_COMMAND_LIST_SUPPORT_FLAG_COMPUTE 0xF000001BL
#define MSG_Map_D3D12_COMMAND_LIST_SUPPORT_FLAG_COPY 0xF000001CL
#define MSG_Map_D3D12_COMMAND_LIST_SUPPORT_FLAG_VIDEO_DECODE 0xF000001DL
#define MSG_Map_D3D12_COMMAND_LIST_SUPPORT_FLAG_VIDEO_PROCESS 0xF000001EL
#define MSG_Map_D3D12_PROTECTED_RESOURCE_SESSION_FLAG_NONE 0xF000001FL
#define MSG_Map_D3D12_SHADER_CACHE_FLAG_NONE 0xF0000020L
#define MSG_Map_D3D12_SHADER_CACHE_FLAG_DRIVER_VERSIONED 0xF0000021L
#define MSG_Map_D3D12_SHADER_CACHE_FLAG_USE_WORKING_DIR 0xF0000022L

